/* Memory allocation and deallocation routines */

# include <stdio.h>
# include <stdlib.h>
# include <math.h>

# include "global.h"
# include "rand.h"

/* Function to allocate memory to a population */
void allocate_memory_pop (population *pop, int size)
{
    int i;

    pop->ind = (individual *) malloc (size * sizeof(individual));
    for (i = 0; i < size; i++)
    {
        allocate_memory_ind (&(pop->ind[i]));
    }
    return;
}

/* Function to allocate memory to an individual */
void allocate_memory_ind (individual *ind)
{
    if (nreal != 0)
    {
        ind->xreal = (double *) malloc (nreal*sizeof(double));
    }    
    ind->obj = (double *) malloc (nobj * sizeof(double));
    return;
}

/* Function to deallocate memory to a population */
void deallocate_memory_pop (population *pop, int size)
{
    int i;

    for (i = 0; i < size; i++)
    {
        deallocate_memory_ind (&(pop->ind[i]));
    }
    free (pop->ind);
    return;
}

/* Function to deallocate memory to an individual */
void deallocate_memory_ind (individual *ind)
{
    if (nreal != 0)
    {
        free (ind->xreal);
    }
    free (ind->obj);
    return;
}

/* allocate memory */
void allocate_memory (double **front[], int noPoints)
{
	int i;

	*front = malloc (noPoints * sizeof(double *));
	if (front == NULL)
		ERROR ("memory allocation failed");
	for (i = 0; i < noPoints; i++)
	{
		(*front)[i] = malloc (nobj * sizeof(double));
		if ((*front)[i] == NULL)
			ERROR ("memory allocation failed");
	}
}

/* deallocate memory */
void Deallocate_memrory (double *front[], int noPoints)
{
	int i;

	if (front != NULL)
	{
		for (i = 0; i < noPoints; i++)
		{
			if (front[i] != NULL)
				free (front[i]);
		}
		free (front);
	}
}