/* This file contains the variable and function declarations */

# ifndef _GLOBAL_H_
# define _GLOBAL_H_

# define INF 1.0e14
# define EPS 1.0e-14
# define E  2.71828182845905
# define PI 3.14159265358979

/* set the reference point as externer variable */
extern const double reference[];

/* ERROR Process */
#define ERROR(x) fprintf(stderr, x), fprintf(stderr, "\n"), exit(1)

typedef struct
{
    double *xreal;
    double *obj;
	double fitness;
}
individual;

typedef struct
{
    individual *ind;
}
population;

typedef struct lists
{
    int index;
	int index2;
    struct lists *parent;
    struct lists *child;
}
list;

extern int k;
extern int l;
extern int dimension;
extern int nreal;
extern int nobj;
extern int popsize;
extern int archive_size;
extern double pcross_real;
extern double pmut_real;
extern double eta_c;
extern double eta_m;
extern int nrealmut;
extern int nrealcross;
extern double *min_realvar;
extern double *max_realvar;
extern double *min_binvar;
extern double *max_binvar;
extern double *min_pf;				/* Pareto frontmininmal߽ */
extern double *max_pf;				/* Pareto frontmaximal߽ */

/* 14/12/2010 */
extern double jumping_percentage;	// ȷtransposonı
extern int no_trans;				// transposon
extern double jumping_rate;			// 趨jumpingı
extern int jumping_count;			// jumping

/* 14/12/2010 */
extern int stop_index;				// ʾjumpingȺλ


void allocate_memory (double **front[], int noPoints);
void Deallocate_memrory (double *front[], int noPoints);
void allocate_memory_pop (population *pop, int size);
void allocate_memory_ind (individual *ind);
void deallocate_memory_pop (population *pop, int size);
void deallocate_memory_ind (individual *ind);

int check_dominance (individual *a, individual *b);

void evaluate_pop (population *pop,int size);
void evaluate_ind (individual *ind);

void initialize_pop (population *pop);
void initialize_ind (individual *ind);

void insert (list *node, int x);
list* del (list *node);

void merge(population *pop1, int size1,population *pop2, int size2,population *pop3);
void copy_ind (individual *ind1, individual *ind2);
void copy_pop (population *pop1, population *pop2, int size);

void matingSelection(population *pop1,int size1,population *pop2,int size2);
void selection (population *old_pop,int size1, population *new_pop,int size2);
individual* tournament (individual *ind1, individual *ind2);
void mutation_pop (population *pop,int size);
void real_mutate_ind (individual *ind);
void crossover (individual *parent1, individual *parent2, individual *child1, individual *child2);
void realcross (individual *parent1, individual *parent2, individual *child1, individual *child2);

void test_problem (double *xreal, double *obj);

void calcFitnesses (population *pop,int size);
void calcFitnesses_Jump (population *pop, int size);

double Euclidean_Distance(individual *ind1,individual *ind2);

double search_k_minimum(double *a,int k,int size);

int environmentalSelection_jump (population *pop1,int size1,population *pop2,int size2, double *time1, population *jumping_pop);
void environmentalSelection (population *pop1,int size1,population *pop2,int size2, double *time1);

void truncation_pop (population *pop1,int size1,population *pop2,int size2);
void truncation_pop_jump (population *pop1, int size1, population *pop2, int size2, population *jumping_pop, int jump_size);

void report_pop (population *pop, FILE *fpt);

/* performance metrics */
void get_test_points (double **obj,int num);
double get_gd (population *pop);
double get_igd (population *pop);
double get_hv (population *pop);
double uniform_diversity (population *pop);

/* hypervolume */
double CalculateHypervolume (double *front[], int noPoints, int noObjectives);
double CalculateArea (double *front[], int noPoints, int noObjectives);
double CalculateArea_2D (double *front[], int noPoints, int noObjectives);
int Is_In_Set (double *temp[], int temp_length, double *point);
int Dominates (double *point1, double *point2, int noObjectives);
int Check_Dominate (double *front[], int noPoints, int objective);
int FilterNondominatedSet (double *front[], int noPoints, int noObjectives);
double getBound (double *front[], int noPoints, int i, int j);
void Swap (double *front[], int i, int j);
void copy (double *temp[], double *front[], int i, int j);
int IsRepeaded (int x, int array[], int size);
int ReduceFront(double *front[], int noPoints, int noObjectives, double *slice_depth, int slice_number);
void Sort (double *front[], int noPoints, int objective);
static void Merge (double *front[], int p, int q, int r, int objectives);
void Merge_Sort (double *front[], int p, int r, int objectives);

/* 09/07/19 */
void gd_igd_wfg(population *pop, int front_size);
void gd_igd_uf(population *pop, int front_size);

/* 13/12/2010 */
void JumpingOperation (population *pop, population *new_pop, population *jumped_pop, int popsize);
void CutPasteSame (individual *ind);
void CutPasteDiff (individual *ind1, individual *ind2);
void CopyPasteSame (individual *ind);
void CopyPasteDiff (individual *ind1, individual *ind2);
int IsDuplicate (int *trans_position, int check_position, int end);
int IsInTransposon (int *trans_position, int check_position);

/* 14/12/2010 */
int Filter_Nondominated (population *mixed_pop, population *nondominated_pop);
int IsDifferent (individual *ind1, individual *ind2);
void Filter_Unelite (population *nondominated_pop, population *archive, population *jumping_pop, int no_nondominated);

/* 2011/01/01 */
void score_pop (population *pop_final);
void gd_igd_zdt3 (population *pop);
void gd_igd_dtlz7 (population *pop);

# endif